module Main where

import Conduit
import Control.Monad.Reader (runReaderT)
import qualified Data.Text as Text
import Options.Applicative

import PUBMED (getMetadataWith, getMetadataWithC)
import PUBMED.Types


data MetaParams = MetaParams
  { mp_mAPIKey :: Maybe String
  , mp_term    :: String }

data FetchParams = FetchParams
  { fp_mAPIKey :: Maybe String
  , fp_limit   :: Int
  , fp_term    :: String }

data Command
  = Meta MetaParams
  | Fetch FetchParams

metaParams :: Parser Command
metaParams = Meta <$>
  (MetaParams
   <$> optional (strOption
                 ( long "api-key"
                   <> metavar "API_KEY"
                   <> help "Pubmed API key"))
   <*> strArgument (metavar "term"))

fetchParams :: Parser Command
fetchParams = Fetch <$>
  (FetchParams
   <$> optional (strOption
                 ( long "api-key"
                   <> metavar "API_KEY"
                   <> help "Pubmed API key"))
    <*> option auto
    ( long "limit"
      <> help "Maximum number of documents"
      <> showDefault
      <> value 100
      <> metavar "INT" )
    <*> strArgument (metavar "term"))

params :: Parser Command
params = subparser
  (command "meta" (info metaParams (progDesc "Download only metadata"))
   <> command "fetch" (info fetchParams (progDesc "Fetch docs")))

opts :: ParserInfo Command
opts = info (params <**> helper)
       (fullDesc
        <> progDesc "A program to test PUBMD"
         <> header "crawlerPubMed-exe")

main :: IO ()
main = run =<< execParser opts
  where

run :: Command -> IO ()
run (Meta (MetaParams { mp_mAPIKey, mp_term })) = do
  let config = Config { apiKey = Text.pack <$> mp_mAPIKey
                      , query = Text.pack mp_term
                      , perPage = Nothing }

  eDocs <- runReaderT getMetadataWithC config
  case eDocs of
    Left err -> print err
    Right (count, _docsC) -> print $ show count
run (Fetch (FetchParams { fp_mAPIKey, fp_term, fp_limit })) = do
  let config = Config { apiKey = Text.pack <$> fp_mAPIKey
                      , query = Text.pack fp_term
                      , perPage = Nothing }

  eDocs <- runReaderT getMetadataWithC config
  case eDocs of
    Left err -> print err
    Right (count, docsC) -> do
      print $ show count
      runConduit $ docsC .| takeC fp_limit .| mapM_C print
